/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.usermanager;

import java.util.Map;



public interface Realm {

	/**
	 * Returns the realm configuration bean.
	 * @throws UserManagerException TODO
	 */
	public Object getRealmConfiguration() throws UserManagerException;
	
	/**
	 * Initializes the realm using the properties from the configuration bean passed in.
	 * @throws UserManagerException TODO
	 */
	public void init(Object configBean) throws UserManagerException;

	/**
	 * Add a user to the userstore.
	 */
	public void addUser(String userName, Object credential) throws UserManagerException;

	/**
	 * Updates a user in the userstore.
	 * Updates his password and properties. 
	 */
	public void updateUser(String userName, Object credential) throws UserManagerException;

	/**
	 * Deletes a user in the userstore
	 */
	public void deleteUser(String userName) throws UserManagerException;
	
	/**
	 * Gets user property
	 */
	public Map getUserProperties(String userName) throws UserManagerException;
	
	/**
	 * Sets user property 
	 */
	public void setUserProperties(String userName, Map properties)  throws UserManagerException;
	
	/**
	 * Returns an array of Usernames from the userstore
	 */
	public String[] getAllUserNames() throws UserManagerException;

	/**
	 * Add a role to the userstore
	 */
	public void addRole(String roleName) throws UserManagerException;

	/**
	 * Delete role from userstore given the rolename.
	 */
	public void deleteRole(String roleName) throws UserManagerException;
	
	/**
	 * Gets role property
	 */
	public Map getRoleProperties(String roleName)  throws UserManagerException;
	
	/**
	 * Sets role property 
	 */
	public void setRoleProperties(String roleName, Map properties)  throws UserManagerException;
	
	/**
	 * Returns an array of all rolenames int the userstore
	 */
	public String[] getAllRoleNames() throws UserManagerException;

	/**
	 * Returns an array of Role given a user name
	 */
	public String[] getUserRoles(String userName)  throws UserManagerException;

	/**
	 * Adds a User to the Role
	 */
	public void addUserToRole(String userName, String roleName) throws UserManagerException;

	/**
	 * Removes a user from role
	 */
	public void removeUserFromRole(String userName, String roleName) throws UserManagerException;

	/**
	 * Authenticates a user against a userstore
	 */
	public boolean authenticate(String userName, Object credentials) throws UserManagerException;

	/**
	 * Allows the user to perform the given action on the given resource
	 */
	public void authorizeUser(String userName, String resourceId, String action) throws UserManagerException;

	/**
	 * Blocks the user performing the given action on the given resource
	 */
	public void denyUser(String userName, String resourceId, String action) throws UserManagerException;

	/**
	 * Checks whether the user is allowed to perform the given action on the given resource
	 */
	public boolean isUserAuthorized(String userName, String resourceId, String action) throws UserManagerException;

	/**
	 * Removes the "user, action, resource" relationship. 
	 * User, action and resource will exist, but the relationship will be removed.
	 */
	public void clearUserAuthorization(String userName, String resourceId, String action) throws UserManagerException;

	/**
	 * Allows the role to perform the given action on the given resource
	 */
	public void authorizeRole(String roleName, String resourceId, String action) throws UserManagerException;

	/**
	 * Blocks the role performing the given action on the given resource
	 */
	public void denyRole(String roleName, String resourceId, String action)throws UserManagerException;

	/**
	 * Checks whether the role is allowed to perform the given action on the given resource
	 */
	public boolean isRoleAuthorized(String roleName, String resourceId, String action) throws UserManagerException;

	/**
	 * Removes the "role, action, resource" relationship. 
	 * Role, action and resource will exist, but the relationship will be removed.
	 */
	public void clearRoleAuthorization(String roleName, String resourceId, String action) throws UserManagerException;

	/**
	 * Removes all authorizations specified on a resource.
	 */
	public void clearResourceAuthorizations(String resourceId)  throws UserManagerException;

	/**
	 * Returns a list of users authorized to perform an action on a resource
	 */
	public String[] getAuthorizedUsersForResource(String resourceId, String action) throws UserManagerException;

	/**
	 * Returns a list of roles authorized to perform an action on a resource
	 */
	public String[] getAuthorizedRolesForResource(String resourceId, String action) throws UserManagerException;

	/**
	 * Copies all authorization of "fromResourceId" to "toResourceId".
	 * All means all - including user authorizations and role authorizations.
	 */
	public void copyAuthorizations (String fromResourceId, String toResourceId) throws UserManagerException;
	
}
